import { jsx as _jsx, Fragment as _Fragment } from "react/jsx-runtime";
import { useCallback, useContext, useEffect, useState } from 'react';
import { ErrorContext } from '../hooks/error';
import { useRequest, useSSE } from '../tools';
import { DefaultNotFound, DefaultTransition } from '../Defaults';
import { ConfigContext } from '../hooks/config';
import { usePageEventListen } from '../events';
import { EventContextProvider, useEventContext } from '../hooks/eventContext';
import { LocationContext } from '../hooks/locationContext';
import { AnyCompList } from './index';
import { SpinnerComp } from './spinner';
export const ServerLoadComp = (props) => {
    const { path, components, loadTrigger, sse, method, sseRetry } = props;
    if (components) {
        return (_jsx(ServerLoadDefer, { path: path, components: components, loadTrigger: loadTrigger, sse: sse, method: method, sseRetry: sseRetry }));
    }
    else if (sse) {
        return _jsx(ServerLoadSSE, { path: path, method: method, sseRetry: sseRetry });
    }
    else {
        return _jsx(ServerLoadFetch, { path: path });
    }
};
const ServerLoadDefer = ({ components, path, loadTrigger, sse, method, sseRetry }) => {
    const { eventContext } = usePageEventListen(loadTrigger);
    if (eventContext) {
        return (_jsx(EventContextProvider, { context: eventContext, children: sse ? (_jsx(ServerLoadSSE, { path: path, method: method, sseRetry: sseRetry })) : (_jsx(ServerLoadFetch, { path: path, method: method })) }));
    }
    else {
        return _jsx(AnyCompList, { propsList: components });
    }
};
export const ServerLoadFetch = ({ path, devReload, method }) => {
    const [transitioning, setTransitioning] = useState(false);
    const [componentProps, setComponentProps] = useState(null);
    const [notFoundUrl, setNotFoundUrl] = useState(undefined);
    const { fullPath } = useContext(LocationContext);
    const url = useServerUrl(path);
    const request = useRequest();
    useEffect(() => {
        setTransitioning(true);
        let componentLoaded = true;
        request({ url, method, expectedStatus: [200, 345, 404] }).then(([status, data]) => {
            if (componentLoaded) {
                // 345 is treat the same as 200 - the server is expected to return valid FastUI components
                if (status === 200 || status === 345) {
                    setComponentProps(data);
                    // if there's a fragment, scroll to that ID once the page is loaded
                    const fragment = getFragment(path);
                    if (fragment) {
                        setTimeout(() => {
                            const element = document.getElementById(fragment);
                            if (element) {
                                element.scrollIntoView();
                            }
                        }, 50);
                    }
                }
                else {
                    setNotFoundUrl(url);
                }
            }
            setTransitioning(false);
        });
        return () => {
            componentLoaded = false;
        };
    }, [url, path, request, devReload, method]);
    useEffect(() => {
        setNotFoundUrl(undefined);
    }, [fullPath]);
    return _jsx(Render, { propsList: componentProps, notFoundUrl: notFoundUrl, transitioning: transitioning });
};
export const ServerLoadSSE = ({ path, method, sseRetry }) => {
    const [componentProps, setComponentProps] = useState(null);
    const url = useServerUrl(path);
    const onMessage = useCallback((data) => setComponentProps(data), []);
    useSSE(url, onMessage, method, sseRetry);
    return _jsx(Render, { propsList: componentProps, transitioning: false });
};
const Render = ({ propsList, notFoundUrl, transitioning, }) => {
    const { error } = useContext(ErrorContext);
    const { NotFound, Transition } = useContext(ConfigContext);
    const NotFoundComp = NotFound ?? DefaultNotFound;
    const TransitionComp = Transition ?? DefaultTransition;
    if (notFoundUrl) {
        return _jsx(NotFoundComp, { url: notFoundUrl });
    }
    else if (propsList === null) {
        if (error) {
            return _jsx(_Fragment, {});
        }
        else {
            return _jsx(SpinnerComp, { type: "Spinner" });
        }
    }
    else {
        return (_jsx(TransitionComp, { transitioning: transitioning, children: _jsx(AnyCompList, { propsList: propsList }) }));
    }
};
function useServerUrl(path) {
    const { APIRootUrl, APIPathMode, APIPathStrip } = useContext(ConfigContext);
    if (APIPathStrip && path.startsWith(APIPathStrip)) {
        path = path.slice(APIPathStrip.length);
    }
    const applyContext = useEventContext();
    const requestPath = applyContext(path);
    if (APIPathMode === 'query') {
        return `${APIRootUrl}?path=${encodeURIComponent(requestPath)}`;
    }
    else {
        return APIRootUrl + requestPath;
    }
}
function getFragment(path) {
    const index = path.indexOf('#');
    if (index !== -1) {
        return path.slice(index + 1);
    }
}
