import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useContext, useState, useRef, useCallback, useEffect } from 'react';
import { useClassName } from '../hooks/className';
import { useRequest } from '../tools';
import { LocationContext } from '../hooks/locationContext';
import { usePageEventListen } from '../events';
import { AnyCompList } from './index';
import { ButtonComp } from './button';
export const FormComp = (props) => {
    const formRef = useRef(null);
    const { formFields, initial, submitUrl, method, footer, displayMode, submitOnChange, submitTrigger, loading: loadingComponents, } = props;
    // mostly equivalent to `<input disabled`
    const [locked, setLocked] = useState(false);
    const [fieldErrors, setFieldErrors] = useState({});
    const [error, setError] = useState(null);
    const [responseComponentProps, setResponseComponentProps] = useState(null);
    // if form fields change or the submit url changes, clear the response
    useEffect(() => {
        setResponseComponentProps(null);
    }, [formFields, submitUrl]);
    const request = useRequest();
    const { goto } = useContext(LocationContext);
    const submit = useCallback(async (formData) => {
        setLocked(true);
        setError(null);
        setFieldErrors({});
        if (method === 'GOTO') {
            // this seems to work in common cases, but typescript doesn't like it
            const query = new URLSearchParams(formData);
            for (const [k, v] of query.entries()) {
                if (v === '') {
                    query.delete(k);
                }
            }
            const queryStr = query.toString();
            goto(queryStr === '' ? submitUrl : `${submitUrl}?${queryStr}`);
            setLocked(false);
            return;
        }
        const requestArgs = { url: submitUrl, expectedStatus: [200, 422] };
        if (method === 'GET') {
            // as above with URLSearchParams
            requestArgs.query = new URLSearchParams(formData);
        }
        else {
            requestArgs.formData = formData;
        }
        const [status, data] = await request(requestArgs);
        if (status === 200) {
            setResponseComponentProps(data);
        }
        else {
            console.assert(status === 422);
            const errorResponse = data;
            const formErrors = errorResponse.detail.form;
            if (formErrors) {
                setFieldErrors(Object.fromEntries(formErrors.map((e) => [locToName(e.loc), e.msg])));
            }
            else {
                console.warn('Non-field error submitting form:', data);
                setError('Error submitting form');
            }
        }
        setLocked(false);
    }, [goto, method, request, submitUrl]);
    const onSubmit = async (e) => {
        e.preventDefault();
        const formData = new FormData(e.currentTarget);
        await submit(formData);
    };
    const onChange = useCallback(() => {
        submitOnChange && formRef.current && formRef.current.requestSubmit();
    }, [submitOnChange]);
    const { fireId } = usePageEventListen(submitTrigger);
    useEffect(() => {
        fireId && formRef.current && formRef.current.requestSubmit();
    }, [fireId]);
    const fieldProps = formFields.map((formField) => {
        const f = {
            ...formField,
            error: fieldErrors[formField.name],
            locked,
            displayMode,
            onChange,
        };
        const formInitial = initial && initial[formField.name];
        if (formInitial !== undefined) {
            ;
            f.initial = formInitial;
        }
        return f;
    });
    const containerClassName = useClassName(props, { el: 'form-container' });
    const formClassName = useClassName(props);
    if (responseComponentProps) {
        return (_jsx("div", { className: containerClassName, children: _jsx(AnyCompList, { propsList: responseComponentProps }) }));
    }
    else {
        return (_jsx("div", { className: containerClassName, children: _jsxs("form", { ref: formRef, className: formClassName, onSubmit: onSubmit, children: [locked && loadingComponents && _jsx(AnyCompList, { propsList: loadingComponents }), _jsx(AnyCompList, { propsList: fieldProps }), error ? _jsxs("div", { children: ["Error: ", error] }) : null, _jsx(Footer, { footer: footer })] }) }));
    }
};
const Footer = ({ footer }) => {
    if (typeof footer === 'undefined') {
        return _jsx(ButtonComp, { type: "Button", text: "Submit", htmlType: "submit" });
    }
    else {
        return _jsx(AnyCompList, { propsList: footer });
    }
};
/**
 * Should match `loc_to_name` in python so errors can be connected to the correct field
 */
function locToName(loc) {
    if (loc.some((v) => typeof v === 'string' && v.includes('.'))) {
        return JSON.stringify(loc);
    }
    else if (typeof loc[0] === 'string' && loc[0].startsWith('[')) {
        return JSON.stringify(loc);
    }
    else {
        return loc.join('.');
    }
}
